@@ -27,6 +27,8 @@ use crate::Event;
 
 pub(crate) mod anonymize;
 pub(crate) mod collect_variants;
+pub(crate) mod comparison;
+pub(crate) mod log2_fold_change;
 
 pub(crate) use collect_variants::collect_variants;
 
